%% Loopshaping Components
% This file generates Bode magnitude plots (straight-line approximations) 
% for the various controller components used in loopshaping design.


%% Component 1: Proportional Gain
Kp = 10;
KpdB = 20*log10(Kp);

ymin = -10;
ymax = 30;

figure(1)
clf
semilogx([0.1 1 10],KpdB*[1 1 1],'b');
legend('$K_p = 10$','Location','NorthEast');
ylim([ymin ymax])
xlabel('Frequency, rad/sec');
ylabel('Magnitude, dB');
set(gca, 'LooseInset', get(gca, 'TightInset'));

if exist('garyfyFigure','file') && exist('matlab2tikz','file');
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    
    matlab2tikz('Kp.tikz','width','\figurewidth',...
        'parseStrings',false,'extraAxisOptions',...
        'ylabel near ticks, xlabel near ticks');
end


%% Components 2 and 3: Low Frequency Boost and Integral Boost
wbar = 0.1;
beta = 100;
betadB = 20*log10(beta);

ymin = -10;
ymax = 60;

figure(2)
clf
semilogx([wbar/(10*beta)  wbar/beta wbar 10*wbar],...
    [betadB betadB 1 1],'b');
hold on;
semilogx([wbar/(10*beta)  wbar/beta wbar 10*wbar],...
    [betadB+20 betadB 1 1],'r--');
legend('$(\bar\omega,\beta) = (0.1,100)$',...
    '$\bar\omega = 0.1$','Location','NorthEast');
ph(1) = plot(wbar/beta*[1 1],[ymin betadB],'k:');
ph(2) = plot(wbar*[1 1],[ymin 0],'k:');
hold off;
ylim([ymin ymax])
xlabel('Frequency, rad/sec');
ylabel('Magnitude, dB');
set(gca, 'LooseInset', get(gca, 'TightInset'));
th(1) = text(1.2e-3,-5,'\frac{\bar \omega}{\beta}');
th(2) = text(1.2e-1,-5,'\bar \omega');

if exist('garyfyFigure','file') && exist('matlab2tikz','file');
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    for i=1:numel(th)
        set(th(i),'String',['$' get(th(i),'String') '$']);
    end
    
    matlab2tikz('Klf.tikz','width','\figurewidth',...
        'parseStrings',false,'extraAxisOptions',...
        'ylabel near ticks, xlabel near ticks');
end

%% Component 4: Roll-off
wbar = 10;

ymin = -20;
ymax = 10;

figure(3)
clf
semilogx([wbar/10  wbar 10*wbar],[0 0 -20],'b');
hold on;
legend('$\bar\omega = 10$','Location','NorthEast');
ph(1) = plot(wbar*[1 1],[ymin 0],'k:');
hold off;
ylim([ymin ymax])
xlabel('Frequency, rad/sec');
ylabel('Magnitude, dB');
set(gca, 'LooseInset', get(gca, 'TightInset'));
th(1) = text(6,-15,'\bar \omega');

if exist('garyfyFigure','file') && exist('matlab2tikz','file');
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    for i=1:numel(th)
        set(th(i),'String',['$' get(th(i),'String') '$']);
    end
    
    matlab2tikz('Kro.tikz','width','\figurewidth',...
        'parseStrings',false,'extraAxisOptions',...
        'ylabel near ticks, xlabel near ticks');
end


%% Components 5: Lead
wbar = 1;
beta = 10;
betadB = 20*log10(beta);

ymin = -30;
ymax = 30;

figure(4)
clf
semilogx([wbar/(10*beta)  wbar/beta beta*wbar 10*beta*wbar],...
    betadB*[-1 -1 1 1],'b');
hold on;
legend('$(\bar\omega,\beta) = (1,10)$','Location','NorthWest');
ph(1) = plot(wbar/beta*[1 1],[ymin -betadB],'k:');
ph(2) = plot(wbar*[1 1],[ymin 0],'k:');
ph(3) = plot(10*wbar*[1 1],[ymin betadB],'k:');
hold off;
ylim([ymin ymax])
xlabel('Frequency, rad/sec');
ylabel('Magnitude, dB');
set(gca, 'LooseInset', get(gca, 'TightInset'));
th(1) = text(1.2e-1,-25,'\frac{\bar \omega}{\beta}');
th(2) = text(1.2,-25,'\bar \omega');
th(3) = text(12,-25,'\beta \bar \omega');

if exist('garyfyFigure','file') && exist('matlab2tikz','file');
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    for i=1:numel(th)
        set(th(i),'String',['$' get(th(i),'String') '$']);
    end
    
    matlab2tikz('Klead.tikz','width','\figurewidth',...
        'parseStrings',false,'extraAxisOptions',...
        'ylabel near ticks, xlabel near ticks');
end
