%% Effect of Loopshaping Components
% This file studies the effects of various loopshaping components.

%% Frequency vector
Nw = 1000;
w = logspace(-2,2,Nw)';

%% Plant
Gnum = 5;
Gden = [1 2];
G = tf(Gnum,Gden);

Gmag = bode(G,w);
GmagdB = 20*log10(Gmag(:));

%% Specify reference and noise signals for simulations
N = 1000;
Tf = 12;
tData = linspace(0,Tf,N);
tData = tData(:);

% Slow ramp up to a constant
idx1 = tData<2;
idx2 = tData>=2 & tData<=8;
idx3 = tData>8;
rData = zeros(N,1);
rData(idx1) = 1-cos(pi*tData(idx1)/2);
rData(idx2) = 2;
%rData(idx3) = 1+cos( pi*(tData(idx3)-8)/2 );
rData(idx3) = 2;
rData = [tData rData];

% Reset random number generator
% This allows the results\ for the random signal to be repeatable.
stream = RandStream.getGlobalStream;
reset(stream);

% Random noise
nData = 0.5*randn(N,1);
nData = [tData nData];


%% Generate results for each controller
Ncomp = 4;          % Number of components
for i=1:Ncomp
    %% Controller K and Loop L=GK
    if i==1
        % Unity feedback
        Knum = 1;
        Kden = 1;
    elseif i==2
        % (Non-unity) Proportional Gain
        Kp = 3;
        Knum = Kp;
        Kden = 1;
    elseif i==3
        % Integral Boost
        wbar = 0.5;
        Knum = [1 wbar];
        Kden = [1 0];
    elseif i==4
        % Rolloff
        wbar = 10;
        Knum = wbar;
        Kden = [1 wbar];
    end
    K = tf(Knum,Kden);
    L = G*K;
    
    %% Bode Magnitudes of S, T, and L
    Kmag = bode(K,w);
    KmagdB = 20*log10(Kmag(:));
    
    Lmag = bode(L,w);
    LmagdB = 20*log10(Lmag(:));
    
    T = feedback(L,1);
    Tmag = bode(T,w);
    TmagdB = 20*log10(Tmag(:));
    
    S = feedback(1,L);
    Smag = bode(S,w);
    SmagdB = 20*log10(Smag(:));
    
    % Plot |S|, |T| and |L|
    figure(2*i-1);
    clf
    if i==1
        subplot(2,1,1)
        semilogx(w,LmagdB,'k:');
        axis([w(1) w(end) -30 10]);
        ylabel('Magnitude, dB');
        xlabel('Frequency, rad/sec');
        legend('|G|','Location','Best');
        set(gca,'YTick',-30:10:10);
        grid on;

        subplot(2,1,2)
        semilogx(w,SmagdB,'b',w,TmagdB,'r-.');
        axis([w(1) w(end) -30 10]);
        ylabel('Magnitude, dB');
        xlabel('Frequency, rad/sec');
        legend('|S|','|T|','Location','Best');
        set(gca,'YTick',-30:10:10);
        grid on;
    else
        ph1 = semilogx(w,LmagdB,'r');
        hold on;
        semilogx(w,KmagdB,'b-.',w,GmagdB,'k:');
        hold off;
        axis([w(1) w(end) -40 40]);
        ylabel('Magnitude, dB');
        xlabel('Frequency, rad/sec');
        legend('|L|=|GK|','|K|','|G|','Location','Best');
        set(gca,'YTick',-40:20:40);
        set(gca, 'LooseInset', get(gca, 'TightInset'));
        grid on;
    end
    
    %% Simulate Closed Loop with Reference and Noise
    sim('LSComponentsExample',Tf);
    if i==1
        t1 = t; y1 = y;
    end

    figure(2*i);
    clf;
    if i==1
        plot(t,r,'b',t,y,'k:')
        legend('Reference, r','Output, y','Location','Best');
    else
        plot(t,r,'b',t,y,'r-.')
        hold on
        ph2 = plot(t1,y1,'k:');
        hold off;
        legend('Reference, r','Output, y: L=GK','Output, y: L=G',...
            'Location','Best');
    end
    grid
    xlabel('Time, sec');
    ylabel('Output, y');
    axis([0 Tf -0.5 2.5]);
    set(gca, 'LooseInset', get(gca, 'TightInset'));
    if i==4
        legend('Reference, r','Output, y: L=GK','Output, y: L=G',...
            'Location','Northeast');

        % Add inset figure to zoom in on the noise
        axes('position',[0.4 0.28 0.52 0.31])
        plot(t,y,'r-.');
        hold on;
        ph3 = plot(t1,y1,'k:');
        hold off;
        grid
        xlabel('Time, sec');
        ylabel('Output, y');
        axis([3 10 1.2 1.7]);
    end

    %% Save Plots
    if exist('garyfyFigure','file') && exist('myprint','file')
        % Code to clean up and save figure.
        % These functions are not in the standard Matlab release.
        figure(2*i-1)
        garyfyFigure
        if i>1
            set(ph1,'LineWidth',5);
        end
        myprint(2*i-1,['LScomp_Lmag' int2str(i)])

        figure(2*i)
        garyfyFigure
        if i>1
            set(ph2,'LineWidth',1.5)
        end
        if i==4
            set(ph3,'LineWidth',1.5);
        end
        myprint(2*i,['LScomp_Resp' int2str(i)])
    end    
end

